from __future__ import division

import numpy as np
import math

from ase.neighborlist import NeighborList
from ase.calculators.calculator import Calculator, all_changes
from ase.calculators.calculator import PropertyNotImplementedError
from ase.calculators.singlepoint import SinglePointCalculator
from mpi4py import MPI

class CombineCalc(Calculator):
    implemented_properties = ['energy', 'forces']

    def __init__(self,calcs=[None], wenergies=[0.5,0.5,0.5,0.5,0.5],wforces=[0.5,0.5,0.5,0.5,0.5],calcforces=[True,True,True,True,True],**kwargs):
        self.calcs=calcs
        self.wenergies=wenergies
        self.wforces=wforces
        self.calcforces=calcforces

        Calculator.__init__(self, **kwargs)

    def calculate(self, atoms=None,
                  properties=['energy','forces'],
                  system_changes=all_changes):
        Calculator.calculate(self, atoms, properties, system_changes)

        energy=0.0
        forces=np.zeros((len(self.atoms),3))
        for i in range(len(self.calcs)):
            a=self.atoms.copy()
            a.set_calculator(self.calcs[i])
            energy+=self.wenergies[i]*a.get_potential_energy()
#            print('calc'+str(i)+' energy = ',self.wenergies[i]*a.get_potential_energy())
            if self.calcforces[i]==True:
                forces+=self.wforces[i]*a.get_forces()

#        print('totalenergy = ',energy)
        self.results['energy'] = energy
        self.results['forces'] = forces

class ParCombineCalc(Calculator):
    implemented_properties = ['energy', 'forces']

    def __init__(self,calcs=[None], wenergies=[0.5,0.5,0.5,0.5,0.5],wforces=[0.5,0.5,0.5,0.5,0.5],calcforces=[True,True,True,True,True],calcpar=[True,True,True,True,True],**kwargs):
        self.calcs=calcs
        self.wenergies=wenergies
        self.wforces=wforces
        self.calcpar=calcpar
        self.calcforces=calcforces
        self.comm = MPI.COMM_WORLD
        self.rank = self.comm.Get_rank()
        self.size = MPI.COMM_WORLD.Get_size()

        Calculator.__init__(self, **kwargs)

    def calculate(self, atoms=None,
                  properties=['energy','forces'],
                  system_changes=all_changes):
        Calculator.calculate(self, atoms, properties, system_changes)

        energy=0.0
        forces=np.zeros((len(self.atoms),3))
        for i in range(len(self.calcs)):
            a=self.atoms.copy()
            a.set_calculator(self.calcs[i])
            if self.calcpar[i]==False:
               if self.rank==0:
                  energy+=self.wenergies[i]*a.get_potential_energy()
#            print('calc'+str(i)+' energy = ',self.wenergies[i]*a.get_potential_energy())
                  if self.calcforces[i]==True:
                     forces+=self.wforces[i]*a.get_forces()
               self.comm.Barrier()
               for k in range(1,self.size):
                   if self.rank==0:
                      self.comm.send(energy,dest=k,tag=303*k)
                      self.comm.send(forces,dest=k,tag=311*k)
                   elif self.rank==k:
                      energy=self.comm.recv(source=0,tag=303*k)
                      forces=self.comm.recv(source=0,tag=311*k)
            else:
                energy+=self.wenergies[i]*a.get_potential_energy()
#            print('calc'+str(i)+' energy = ',self.wenergies[i]*a.get_potential_energy())
                if self.calcforces[i]==True:
                    forces+=self.wforces[i]*a.get_forces()

#        print('totalenergy = ',energy)
        self.results['energy'] = energy
        self.results['forces'] = forces

