import numpy as np

from ase import Atoms
from gpaw import GPAW, Mixer, MixerSum, MixerDif
from ase.io import read
from gofee.candidates import CandidateGenerator, StartGenerator, RattleMutation, PermutationMutation
from gofee import GOFEE
from rdf import RDF,RDFmultiphase
from combine import CombineCalc
import sys

#second fixed phase:
b='a.cif'
#some memory/IO related things
try:
   ramdisk='/ramdisk/'+sys.argv[1]
except:
   ramdisk='.'

# DIFFPY PDF calculator
g='sns2.gr'
biso=[['Sn','S'],[0.2521754,0.452104194]]
calcb = RDFmultiphase(Gr=g, strucs=b, pdfrange=[2.3,155.0],delta=0.01,scale=0.218, qdamp=0.0084,calcforces=False,weightphases=0.8,psize=None,delta2=None,biso=biso,label=ramdisk+'/xxrh')

#DFT Calculator
calca=GPAW(xc='PBE', mode='lcao', kpts=(3,3,2), basis='dzp',symmetry='off',mixer=Mixer(0.02, 5, 100), convergence={'energy': 0.001,'density': 0.20e-2,'eigenstates': 4.0e-7, 'bands': 'occupied'})

#Mixing Calculator
calc=CombineCalc(calcs=[calca,calcb], wenergies=[1.0,3.0],wforces=[1.0,0.0],calcforces=[True,False])

# Define template for the global optimization
template=read('template.traj')
cell=template.cell

# Stoichiometry of atoms to be placed
stoichiometry = 1*[50] + 4*[16] 

# Box in which to place atoms randomly
v = 0.95*cell
p0 = np.array((0, 0, 0))
box = [p0, v]

# initialize startgenerator (used to generate initial structures)
sg = StartGenerator(template, stoichiometry, box)

### Set up candidate generation operations ###
# initialize rattle mutation
n_to_optimize = len(stoichiometry)
rattle = RattleMutation(n_to_optimize, Nrattle=5, rattle_range=4)
rattleb = RattleMutation(n_to_optimize, Nrattle=3, rattle_range=7)
permutation = PermutationMutation(n_to_optimize, Npermute=2)
candidate_generator = CandidateGenerator(probabilities=[0.1, 0.2, 0.3,0.4],
                                         operations=[sg, rattle, rattleb,permutation])

### Initialize and run search ###
search1 = GOFEE(calc=calc,
               startgenerator=sg,
               candidate_generator=candidate_generator,
               max_steps=2000,
               Ncandidates=5,
               population_size=10,
               logfile='-',
               trajectory='structures.traj')

search1.run()
